/**
 * -----------------------------------------------------------------------------
 * @package     smartVISU
 * @author      Martin Gleiß
 * @copyright   2012
 * @license     GPL [http://www.gnu.de]
 * -----------------------------------------------------------------------------
 */


(function ($, undefined) {

	$.widget("mobile.slider", $.mobile.widget, {
		widgetEventPrefix: "slide",

		options: {
			theme: null,
			trackTheme: null,
			disabled: false,
			initSelector: "input[type='range'], :jqmData(type='range'), :jqmData(role='slider')",
			mini: false,
			highlight: false
		},

		_create: function () {

			// TODO: Each of these should have comments explain what they're for
			var self = this,
				control = this.element,
				parentTheme = $.mobile.getInheritedTheme(control, "c"),
				theme = this.options.theme || parentTheme,
				trackTheme = this.options.trackTheme || parentTheme,
				cType = control[ 0 ].nodeName.toLowerCase(),
				isSelect = this.isToggleSwitch = cType === "select",
				isRangeslider = control.parent().is(":jqmData(role='rangeslider')"),
				selectClass = ( this.isToggleSwitch ) ? "ui-slider-switch" : "",
				controlID = control.attr("id"),
				$label = $("[for='" + controlID + "']"),
				labelID = $label.attr("id") || controlID + "-label",
				label = $label.attr("id", labelID),
				min = !this.isToggleSwitch ? parseFloat(control.attr("min")) : 0,
				max = !this.isToggleSwitch ? parseFloat(control.attr("max")) : control.find("option").length - 1,
				orientation = control.attr("orientation"),
				step = window.parseFloat(control.attr("step") || 1),
				miniClass = ( this.options.mini || control.jqmData("mini") ) ? " ui-mini" : "",
				domHandle = document.createElement("a"),
				handle = $(domHandle),
				domSlider = document.createElement("div"),
				slider = $(domSlider),
				valuebg = this.options.highlight && !this.isToggleSwitch ? (function () {
					var bg = document.createElement("div");
					if (orientation == 'semicircle') {
						bg.className = 'ui-slider-bg';
						var bgInner = document.createElement('div');
						bgInner.className = 'ui-slider-track-semicircle ' + $.mobile.activeBtnClass;
						bg.appendChild(bgInner);
					}
					else {
						bg.className = "ui-slider-bg " + $.mobile.activeBtnClass + " ui-btn-corner-all";
					}
					return $(bg).prependTo(slider);
				})() : false,
				options;

			domHandle.setAttribute("href", "#");
			domSlider.setAttribute("role", "application");
			domSlider.className =
				[(this.isToggleSwitch ? "ui-slider" : "ui-slider-track") + (orientation ? "-" + orientation : " "),
					selectClass, " ui-btn-down-", trackTheme, " ui-btn-corner-all", miniClass].join("");
			domHandle.className = "ui-slider-handle" + (orientation ? "-" + orientation : "");
			domSlider.appendChild(domHandle);

			if (orientation == 'semicircle') {
				var domSliderBox = document.createElement('div');
				domSliderBox.className = 'ui-slider-semicircle-box';

				var domSliderCut = document.createElement('div');
				domSliderCut.className = 'ui-slider-semicircle-cut';
				domSliderBox.appendChild(domSliderCut);
				domSliderCut.appendChild(domSlider);

				var domSliderInner = document.createElement('div');
				domSliderInner.className = 'ui-slider-semicircle-inner ui-body-' + trackTheme;
				domSliderCut.appendChild(domSliderInner);

				var domSliderLeft = document.createElement('div');
				domSliderLeft.className = 'ui-slider-semicircle-bottom ' + (valuebg ? $.mobile.activeBtnClass : '') + ' ui-btn-down-' + trackTheme;
				domSliderLeft.style.cssFloat = 'left';
				domSliderBox.appendChild(domSliderLeft);

				var domSliderRight = document.createElement('div');
				domSliderRight.className = 'ui-slider-semicircle-bottom ui-btn-down-' + trackTheme;
				domSliderRight.style.cssFloat = 'right';
				domSliderBox.appendChild(domSliderRight);

				slider = $(domSliderBox);
			}
			;

			handle.buttonMarkup({ corners: true, theme: theme, shadow: true })
				.attr({
					"role": "slider",
					"aria-valuemin": min,
					"aria-valuemax": max,
					"aria-valuenow": this._value(),
					"aria-valuetext": this._value(),
					"title": this._value(),
					"aria-labelledby": labelID
				});

			$.extend(this, {
				slider: slider,
				handle: handle,
				type: cType,
				step: step,
				max: max,
				min: min,
				valuebg: valuebg,
				isRangeslider: isRangeslider,
				dragging: false,
				beforeStart: null,
				userModified: false,
				mouseMoved: false
			});

			if (this.isToggleSwitch) {
				var wrapper = document.createElement("div");
				wrapper.className = "ui-slider-inneroffset";

				for (var j = 0, length = domSlider.childNodes.length; j < length; j++) {
					wrapper.appendChild(domSlider.childNodes[j]);
				}

				domSlider.appendChild(wrapper);

				// slider.wrapInner( "<div class='ui-slider-inneroffset'></div>" );

				// make the handle move with a smooth transition
				handle.addClass("ui-slider-handle-snapping");

				options = control.find("option");

				for (var i = 0, optionsCount = options.length; i < optionsCount; i++) {
					var side = !i ? "b" : "a",
						sliderTheme = !i ? " ui-btn-down-" + trackTheme : ( " " + $.mobile.activeBtnClass ),
						sliderLabel = document.createElement("div"),
						sliderImg = document.createElement("span");

					sliderImg.className =
						["ui-slider-label ui-slider-label-", side, sliderTheme, " ui-btn-corner-all"].join("");
					sliderImg.setAttribute("role", "img");
					sliderImg.appendChild(document.createTextNode(options[i].innerHTML));
					$(sliderImg).prependTo(slider);
				}

				self._labels = $(".ui-slider-label", slider);

			}

			label.addClass("ui-slider");

			// monitor the input for updated values
			control.addClass(this.isToggleSwitch ? "ui-slider-switch" : "ui-slider-input");

			this._on(control, {
				"change": "_controlChange",
				"keyup": "_controlKeyup",
				"blur": "_controlBlur",
				"vmouseup": "_controlVMouseUp"
			});

			slider.bind("vmousedown", $.proxy(this._sliderVMouseDown, this))
				.bind("vclick", false);

			// We have to instantiate a new function object for the unbind to work properly
			// since the method itself is defined in the prototype (causing it to unbind everything)
			this._on(document, { "vmousemove": "_preventDocumentDrag" });
			this._on(slider.add(document), { "vmouseup": "_sliderVMouseUp" });

			slider.insertAfter(control);

			// wrap in a div for styling purposes
			if (!this.isToggleSwitch && !isRangeslider) {
				var wrapper = this.options.mini ? "<div class='ui-slider ui-mini'>" : "<div class='ui-slider'>";

				control.add(slider).wrapAll(wrapper);
			}

			// Only add focus class to toggle switch, sliders get it automatically from ui-btn
			if (this.isToggleSwitch) {
				this.handle.bind({
					focus: function () {
						slider.addClass($.mobile.focusClass);
					},

					blur: function () {
						slider.removeClass($.mobile.focusClass);
					}
				});
			}

			// bind the handle event callbacks and set the context to the widget instance
			this._on(this.handle, {
				"vmousedown": "_handleVMouseDown",
				"keydown": "_handleKeydown",
				"keyup": "_handleKeyup"
			});

			this.handle.bind("vclick", false);

			if (this._handleFormReset) {
				this._handleFormReset();
			}

			this.refresh(undefined, undefined, true);
		},

		_controlChange: function (event) {
			// if the user dragged the handle, the "change" event was triggered from inside refresh(); don't call refresh() again
			if (this._trigger("controlchange", event) === false) {
				return false;
			}
			if (!this.mouseMoved) {
				this.refresh(this._value(), true);
			}
		},

		_controlKeyup: function (event) { // necessary?
			this.refresh(this._value(), true, true);
		},

		_controlBlur: function (event) {
			this.refresh(this._value(), true);
		},

		// it appears the clicking the up and down buttons in chrome on
		// range/number inputs doesn't trigger a change until the field is
		// blurred. Here we check thif the value has changed and refresh
		_controlVMouseUp: function (event) {
			this._checkedRefresh();
		},

		// NOTE force focus on handle
		_handleVMouseDown: function (event) {
			this.handle.focus();
		},

		_handleKeydown: function (event) {
			var index = this._value();
			if (this.options.disabled) {
				return;
			}

			// In all cases prevent the default and mark the handle as active
			switch (event.keyCode) {
				case $.mobile.keyCode.HOME:
				case $.mobile.keyCode.END:
				case $.mobile.keyCode.PAGE_UP:
				case $.mobile.keyCode.PAGE_DOWN:
				case $.mobile.keyCode.UP:
				case $.mobile.keyCode.RIGHT:
				case $.mobile.keyCode.DOWN:
				case $.mobile.keyCode.LEFT:
					event.preventDefault();

					if (!this._keySliding) {
						this._keySliding = true;
						this.handle.addClass("ui-state-active");
					}

					break;
			}

			// move the slider according to the keypress
			switch (event.keyCode) {
				case $.mobile.keyCode.HOME:
					this.refresh(this.min);
					break;
				case $.mobile.keyCode.END:
					this.refresh(this.max);
					break;
				case $.mobile.keyCode.PAGE_UP:
				case $.mobile.keyCode.UP:
				case $.mobile.keyCode.RIGHT:
					this.refresh(index + this.step);
					break;
				case $.mobile.keyCode.PAGE_DOWN:
				case $.mobile.keyCode.DOWN:
				case $.mobile.keyCode.LEFT:
					this.refresh(index - this.step);
					break;
			}
		}, // remove active mark

		_handleKeyup: function (event) {
			if (this._keySliding) {
				this._keySliding = false;
				this.handle.removeClass("ui-state-active");
			}
		},

		_sliderVMouseDown: function (event) {
			// NOTE: we don't do this in refresh because we still want to
			//       support programmatic alteration of disabled inputs
			if (this.options.disabled) {
				return false;
			}
			if (this._trigger("beforestart", event) === false) {
				return false;
			}
			this.dragging = true;
			this.userModified = false;
			this.mouseMoved = false;

			if (this.isToggleSwitch) {
				this.beforeStart = this.element[0].selectedIndex;
			}


			this.refresh(event);
			this._trigger("start");
			return false;
		},

		_sliderVMouseUp: function () {
			if (this.dragging) {
				this.dragging = false;

				if (this.isToggleSwitch) {
					// make the handle move with a smooth transition
					this.handle.addClass("ui-slider-handle-snapping");

					if (this.mouseMoved) {
						// this is a drag, change the value only if user dragged enough
						if (this.userModified) {
							this.refresh(this.beforeStart === 0 ? 1 : 0);
						}
						else {
							this.refresh(this.beforeStart);
						}
					}
					else {
						// this is just a click, change the value
						this.refresh(this.beforeStart === 0 ? 1 : 0);
					}
				}

				this.mouseMoved = false;
				this._trigger("stop");
				return false;
			}
		},

		_preventDocumentDrag: function (event) {
			// NOTE: we don't do this in refresh because we still want to
			//       support programmatic alteration of disabled inputs
			if (this._trigger("drag", event) === false) {
				return false;
			}
			if (this.dragging && !this.options.disabled) {

				// this.mouseMoved must be updated before refresh() because it will be used in the control "change" event
				this.mouseMoved = true;

				if (this.isToggleSwitch) {
					// make the handle move in sync with the mouse
					this.handle.removeClass("ui-slider-handle-snapping");
				}

				this.refresh(event);

				// only after refresh() you can calculate this.userModified
				this.userModified = this.beforeStart !== this.element[0].selectedIndex;
				return false;
			}
		},

		_checkedRefresh: function () {
			if (this.value != this._value()) {
				this.refresh(this._value());
			}
		},

		_value: function () {
			return  this.isToggleSwitch ? this.element[0].selectedIndex : parseFloat(this.element.val());
		},


		_reset: function () {
			this.refresh(undefined, false, true);
		},

		refresh: function (val, isfromControl, preventInputUpdate) {
			// NOTE: we don't return here because we want to support programmatic
			//       alteration of the input value, which should still update the slider

			var self = this,
				parentTheme = $.mobile.getInheritedTheme(this.element, "c"),
				theme = this.options.theme || parentTheme,
				trackTheme = this.options.trackTheme || parentTheme,
				orientation = this.element.attr("orientation");

			if (orientation == 'semicircle') {
				self.slider[0].className = 'ui-slider-semicircle-box';
			}
			else {
				self.slider[0].className = [
					(this.isToggleSwitch ? "ui-slider ui-slider-switch" : "ui-slider-track" + (orientation ? "-" + orientation : " ")),
					" ui-btn-down-" + trackTheme, ' ui-btn-corner-all', ( this.options.mini ) ? " ui-mini" : ""
				].join("");
			}

			if (this.options.disabled || this.element.attr("disabled")) {
				this.disable();
			}

			// set the stored value for comparison later
			this.value = this._value();
			if (this.options.highlight && !this.isToggleSwitch && this.slider.find(".ui-slider-bg").length === 0) {
				this.valuebg = (function () {
					var bg = document.createElement("div");
					bg.className = "ui-slider-bg " + $.mobile.activeBtnClass + " ui-btn-corner-all";
					return $(bg).prependTo(self.slider);
				})();
			}
			this.handle.buttonMarkup({ corners: true, theme: theme, shadow: true });

			var pxStep, percent,
				control = this.element,
				isInput = !this.isToggleSwitch,
				optionElements = isInput ? [] : control.find("option"),
				min = isInput ? parseFloat(control.attr("min")) : 0,
				max = isInput ? parseFloat(control.attr("max")) : optionElements.length - 1,
				step = ( isInput && parseFloat(control.attr("step")) > 0 ) ? parseFloat(control.attr("step")) : 1,
				orientation = control.attr("orientation");

			if (typeof val === "object") {
				var left, width, data = val,
				// a slight tolerance helped get to the ends of the slider
					tol = 8;

				left = this.slider.offset().left;
				width = this.slider.width();
				pxStep = width / ((max - min) / step);
				if (!this.dragging ||
					data.pageX < left - tol ||
					data.pageX > left + width + tol) {
					return;
				}

				if (orientation == "vertical") {
					percent = Math.round(( ( data.pageY - this.slider.offset().top ) / this.slider.height() ) * 100);
				}
				else if (orientation == "bottomup") {
					percent = 100 - Math.round(( ( data.pageY - this.slider.offset().top ) / this.slider.height() ) * 100);
				}
				else if (orientation == "semicircle") {
					var x = ( data.pageX - this.slider.offset().left ) - this.slider.width() / 2 - 2;
					var y = -(( data.pageY - this.slider.offset().top ) - this.slider.height() + 8);
					percent = Math.round(((Math.atan2(x, y) / Math.PI) * 180 + 90) / 180 * 100);
					if (percent < 0) {
						percent = 0;
					}
					if (percent > 100) {
						percent = 100;
					}
				}
				else {
					if (pxStep > 1) {
						percent = ( ( data.pageX - left ) / width ) * 100;
					}
					else {
						percent = Math.round(( ( data.pageX - left ) / width ) * 100);
					}
				}
			}
			else {
				if (val == null) {
					val = isInput ? parseFloat(control.val() || 0) : control[0].selectedIndex;
				}
				percent = ( parseFloat(val) - min ) / ( max - min ) * 100;
			}

			if (isNaN(percent)) {
				return;
			}

			var newval = ( percent / 100 ) * ( max - min ) + min;

			//from jQuery UI slider, the following source will round to the nearest step
			var valModStep = ( newval - min ) % step;
			var alignValue = newval - valModStep;

			if (Math.abs(valModStep) * 2 >= step) {
				alignValue += ( valModStep > 0 ) ? step : ( -step );
			}

			var percentPerStep = 100 / ((max - min) / step);
			// Since JavaScript has problems with large floats, round
			// the final value to 5 digits after the decimal point (see jQueryUI: #4124)
			newval = parseFloat(alignValue.toFixed(5));

			if (typeof pxStep === "undefined") {
				pxStep = width / ( (max - min) / step );
			}
			if (pxStep > 1 && isInput) {
				percent = ( newval - min ) * percentPerStep * ( 1 / step );
			}
			if (percent < 0) {
				percent = 0;
			}

			if (percent > 100) {
				percent = 100;
			}

			if (newval < min) {
				newval = min;
			}

			if (newval > max) {
				newval = max;
			}

			if (orientation == "vertical") {
				this.handle.css("top", percent + "%");
				this.valuebg && this.valuebg.css("height", percent + "%");
			}
			else if (orientation == "bottomup") {
				this.handle.css("top", 100 - percent + "%");
				this.valuebg && this.valuebg.css("margin-top", (this.slider.height() * (100 - percent) / 100).toFixed(0) + 'px');
				this.valuebg && this.valuebg.css("height", percent + "%");
			}

			else if (orientation == "semicircle") {
				var angle = (percent / 100 * 180 - 90 ) / 180 * Math.PI;
				this.handle.css("top", Math.round(-Math.cos(angle) * (this.slider.width() / 2 - 8) + ((this.slider.height() == 0 ? 92 : this.slider.height()) - 4)) + 'px');
				this.handle.css("left", Math.round(Math.sin(angle) * (this.slider.width() / 2 - 8) + (this.slider.width() / 2 + 4)) + 'px');

				var width = (percent * 1.08 - 8).toFixed(0);
				this.valuebg && this.valuebg.css("width", (width < 1 ? 1 : width) + "%");
			}
			else {
				this.handle.css("left", percent + "%");
				this.valuebg && this.valuebg.css("width", percent + "%");
			}

			this.handle[0].setAttribute("aria-valuenow", isInput ? newval : optionElements.eq(newval).attr("value"));

			this.handle[0].setAttribute("aria-valuetext", isInput ? newval : optionElements.eq(newval).getEncodedText());

			this.handle[0].setAttribute("title", isInput ? newval : optionElements.eq(newval).getEncodedText());


			// drag the label widths
			if (this._labels) {
				var handlePercent = this.handle.width() / this.slider.width() * 100,
					aPercent = percent && handlePercent + ( 100 - handlePercent ) * percent / 100,
					bPercent = percent === 100 ? 0 : Math.min(handlePercent + 100 - aPercent, 100);

				this._labels.each(function () {
					var ab = $(this).is(".ui-slider-label-a");
					$(this).width(( ab ? aPercent : bPercent  ) + "%");
				});
			}

			if (!preventInputUpdate) {
				var valueChanged = false;

				// update control"s value
				if (isInput) {
					valueChanged = control.val() !== newval;
					control.val(newval);
				}
				else {
					valueChanged = control[ 0 ].selectedIndex !== newval;
					control[ 0 ].selectedIndex = newval;
				}
				if (this._trigger("beforechange", val) === false) {
					return false;
				}
				if (!isfromControl && valueChanged) {
					control.trigger("change");
				}
			}
		},

		enable: function () {
			this.element.attr("disabled", false);
			this.slider.removeClass("ui-disabled").attr("aria-disabled", false);
			return this._setOption("disabled", false);
		},

		disable: function () {
			this.element.attr("disabled", true);
			this.slider.addClass("ui-disabled").attr("aria-disabled", true);
			return this._setOption("disabled", true);
		}

	});

	$.widget("mobile.slider", $.mobile.slider, $.mobile.behaviors.formReset);

	//auto self-init widgets
	$.mobile.document.bind("pagecreate create", function (e) {
		$.mobile.slider.prototype.enhanceWithin(e.target, true);
	});

})(jQuery);
